package form.comp;

import java.beans.*;
import form.io.*;
import form.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

public class FormTextField extends java.awt.TextField implements FormInterface{


	public boolean getEndOfRow(){
	   return (constraints.gridwidth == 0);
	}

	public void setEndOfRow(boolean eor){
	   if (eor)
	      constraints.gridwidth = 0;
	   else
	      constraints.gridwidth = 1;

	   invalidate();
	   //doLayout();
	   getParent().doLayout();
	   getParent().validate();
	}

	/**
	* The specific constraints for this object
	*/
	private Constraints constraints = new Constraints();

	/**
	* The name of this object.
	* it is (should) be unique over the entire form
	*/
	private String name    = null;

	/**
	* The initial value of the of this object
	* Used to reset the values in the form.
	* otherwise it is basically useless.
	* Is only modified by the setText method
	*/
	private String initVal = null;

	/**
	* Says if this object has the focus of the form
	* window during editing.
	* Is only used when in edit mode
	*/
	private boolean selected = false;

	/**
	* Resets this object to its default value
	* Should only be used in default mode.
	*/
	public void reset(){
		setText(initVal);
	}

	/**
	* Returns the Component repsentation of this object.
	* It is assumed that all FormInterface comps will
	* extend a component.  If not this will help the work around
	* it also simplifiels casting.
	*/
	public Component getComponent(){
		return this;
	}

	/**
	* The default / blank constructor
	* A blank constructor must exist for the creation
	* from the string representation of the class (In Viewer.java)
	*/
	public FormTextField(){	
	      super();
	}

        /**
        * @param String name The name of the object to be created
        * This constructor is used with the insert method during edit.
        */
        public FormTextField(String nam){
                super();
                setName(nam);
                setVisible(true);
                new FormInterfaceListener(this);
	        setBeanProps();

        }

	private void setBeanProps(){
	      try{
		PropertyDescriptor properties[];
		BeanInfo bi = Introspector.getBeanInfo(this.getClass());
		properties = bi.getPropertyDescriptors();
		for (int i = 0; i < properties.length; i++) {

		   if ( (properties[i].getName().equals("selectionStart")) ||
		        (properties[i].getName().equals("caretPosition")) ||
		        (properties[i].getName().equals("selectionEnd")) ||
		        (properties[i].getName().equals("columns")) ||
		        (properties[i].getName().equals("font")) ){
		   	properties[i].setHidden(true);
	           }
	        }
	    }catch (Exception e){e.printStackTrace();}
	}

	/**
	* @param boolean sel True if object is selected
	* used to set the selected object
	*/
	public void setSelect(boolean sel){
		selected = sel;
	}

	/**
	* Used to get and type cast the GridBagConstraints
	* for this object.
	*/
	public GridBagConstraints getGridBagConstraints(){
		return ((GridBagConstraints)constraints);
	}


	/**
	*@param Open open file open object
	* reads the file one line at a time
	* format is:
	* NOTE: update and sync this with the same method
	* The name of this object
	* The initial text
	*/
	public void open(Open open,boolean edit,String name){
		int i =0;

		setName( open.getParameter(name+i++) );

		this.setText( open.getParameter(name+i++) );

		setBackground(new Color(open.getInt(name+i++)) );

		setForeground(new Color(open.getInt(name+i++)) );

		constraints.gridwidth = open.getInt(name+i++);

		setEditable(open.getBoolean(name+i++));

		setVisible(true);
		
		if (edit)
			new FormInterfaceListener(this);

	}

	/**
	* @param Save save The save object which writes
	* the output to the designated location
	* Note: This method should always replicate initialize(...);
	*/
	public void save(ToHtml save,String name){
		int i = 0;
		save.add(name,"form.comp.FormTextField");
		save.add(name+i++,getName());
		save.add(name+i++,getText());
		save.add(name+i++,getBackground().getRGB());
		save.add(name+i++,getForeground().getRGB());
		save.add(name+i++,constraints.gridwidth);
		if(isEditable())
		    save.add(name+i++,"true");
		else save.add(name+i++,"false");
	}

	public void setText(String str){
		initVal = str;
		super.setText(str);
	}

	public String getName(){
		return name;
	}

	public void setName(String str){
		name = str;
	}

	public Dimension getPreferredSize(){
		return new Dimension (50, 30);
	}

	public void addListener(FormActionListener listen){
		addMouseListener(listen);
	}

}
